@@ -3,11 +3,11 @@ Deployment related client handler logic.
 
 Remember that in code snippets all highlighted lines are required other lines are optional.
 """
-from .base_client import BaseClient
+from .base_client import BaseClient, TagsSupportMixin
 from .. import config, models, repositories
 
 
-class DeploymentsClient(BaseClient):
+class DeploymentsClient(TagsSupportMixin, BaseClient):
     """
     Client to handle deployment related actions.
 
@@ -29,10 +29,11 @@ class DeploymentsClient(BaseClient):
     def create(
             self,
             deployment_type,
-            model_id, name,
+            name,
             machine_type,
             image_url,
-            instance_count,
+            instance_count=None,
+            model_id=None,
             container_model_path=None,
             image_username=None,
             image_password=None,
@@ -49,6 +50,12 @@ class DeploymentsClient(BaseClient):
             auth_password=None,
             tags=None,
             command=None,
+            workspace_url=None,
+            workspace_ref=None,
+            workspace_username=None,
+            workspace_password=None,
+            project_id=None,
+            autoscaling=None,
     ):
         """
         Method to create a Deployment instance.
@@ -93,6 +100,12 @@ class DeploymentsClient(BaseClient):
         :param str auth_password: Password
         :param list[str] tags: List of tags
         :param str command: Deployment command
+        :param str workspace_url: Project git or s3repository url
+        :param str workspace_ref: Git commit hash, branch name or tag
+        :param str workspace_username: Project git repository username
+        :param str workspace_password: Project git repository password
+        :param str project_id: Project ID
+        :param models.AutoscalingDefinition autoscaling: Deployment autoscaling definition
 
         :returns: Created deployment id
         :rtype: str
@@ -119,12 +132,18 @@ class DeploymentsClient(BaseClient):
             auth_username=auth_username,
             auth_password=auth_password,
             command=command,
+            workspace_url=workspace_url,
+            workspace_ref=workspace_ref,
+            workspace_username=workspace_username,
+            workspace_password=workspace_password,
+            project_id=project_id,
+            autoscaling=autoscaling,
         )
 
         repository = self.build_repository(repositories.CreateDeployment)
         deployment_id = repository.create(deployment)
         if tags:
-            self.add_tags(entity_id=deployment_id, entity=self.entity, tags=tags)
+            self.add_tags(entity_id=deployment_id, tags=tags)
         return deployment_id
 
     def get(self, deployment_id):
@@ -211,6 +230,13 @@ class DeploymentsClient(BaseClient):
             cluster_id=None,
             auth_username=None,
             auth_password=None,
+            workspace_url=None,
+            workspace_ref=None,
+            workspace_username=None,
+            workspace_password=None,
+            project_id=None,
+            command=None,
+            autoscaling=None,
     ):
         deployment = models.Deployment(
             deployment_type=deployment_type,
@@ -233,7 +259,90 @@ class DeploymentsClient(BaseClient):
             cluster_id=cluster_id,
             auth_username=auth_username,
             auth_password=auth_password,
+            workspace_url=workspace_url,
+            workspace_ref=workspace_ref,
+            workspace_username=workspace_username,
+            workspace_password=workspace_password,
+            project_id=project_id,
+            command=command,
+            autoscaling=autoscaling,
         )
 
         repository = self.build_repository(repositories.UpdateDeployment)
         repository.update(deployment_id, deployment)
+
+    def get_metrics(self, deployment_id, start=None, end=None, interval="30s", built_in_metrics=None):
+        """Get model deployment metrics
+
+        :param str deployment_id: ID of deployment
+        :param datetime.datetime|str start:
+        :param datetime.datetime|str end:
+        :param str interval:
+        :param list[str] built_in_metrics: List of metrics to get if different than default
+                    Available builtin metrics: cpuPercentage, memoryUsage, gpuMemoryFree, gpuMemoryUsed, gpuPowerDraw,
+                                            gpuTemp, gpuUtilization, gpuMemoryUtilization
+
+        :returns: Metrics of a model deployment job
+        :rtype: dict[str,dict[str,list[dict]]]
+        """
+
+        repository = self.build_repository(repositories.GetDeploymentMetrics)
+        metrics = repository.get(
+            id=deployment_id,
+            start=start,
+            end=end,
+            interval=interval,
+            built_in_metrics=built_in_metrics,
+        )
+        return metrics
+
+    def stream_metrics(self, deployment_id, interval="30s", built_in_metrics=None):
+        """Stream live model deployment metrics
+
+        :param str deployment_id: ID of model deployment
+        :param str interval:
+        :param list[str] built_in_metrics: List of metrics to get if different than default
+                    Available builtin metrics: cpuPercentage, memoryUsage, gpuMemoryFree, gpuMemoryUsed, gpuPowerDraw,
+                                            gpuTemp, gpuUtilization, gpuMemoryUtilization
+
+        :returns: Generator object yielding live model deployment metrics
+        :rtype: Iterable[dict]
+        """
+
+        repository = self.build_repository(repositories.StreamDeploymentMetrics)
+        metrics = repository.stream(
+            id=deployment_id,
+            interval=interval,
+            built_in_metrics=built_in_metrics,
+        )
+        return metrics
+
+    def logs(self, deployment_id, line=1, limit=10000):
+        """Show list of latest logs from the specified deployment.
+
+        :param str deployment_id: Deployment Id
+        :param int line: line number at which logs starts to display on screen
+        :param int limit: maximum lines displayed on screen, default set to 10 000
+
+        :returns: list of LogRows
+        :rtype: list[models.LogRow]
+        """
+
+        repository = self.build_repository(repositories.ListDeploymentLogs)
+        logs = repository.list(id=deployment_id, line=line, limit=limit)
+        return logs
+
+    def yield_logs(self, deployment_id, line=1, limit=10000):
+        """Get log generator. Polls the API for new logs
+
+        :param str deployment_id: Deployment Id
+        :param int line: line number at which logs starts to display on screen
+        :param int limit: maximum lines displayed on screen, default set to 10 000
+
+        :returns: generator yielding LogRow instances
+        :rtype: Iterator[models.LogRow]
+        """
+
+        repository = self.build_repository(repositories.ListDeploymentLogs)
+        logs = repository.yield_logs(id=deployment_id, line=line, limit=limit)
+        return logs
