@@ -4,37 +4,89 @@ import click
 
 from gradient import cliutils
 from gradient import exceptions, clilogger, DEPLOYMENT_TYPES_MAP
-from gradient.api_sdk import DeploymentsClient, constants
+from gradient.api_sdk import constants, workspace
+from gradient.api_sdk.s3_uploader import DeploymentWorkspaceDirectoryUploader
 from gradient.cli import common
 from gradient.cli.cli import cli
 from gradient.cli.cli_types import ChoiceType, json_string
 from gradient.cli.common import api_key_option, del_if_value_is_none, ClickGroup, validate_comma_split_option
+from gradient.cli_constants import CLI_PS_CLIENT_NAME
 from gradient.commands import deployments as deployments_commands
 from gradient.commands.deployments import DeploymentRemoveTagsCommand, DeploymentAddTagsCommand, \
-    GetDeploymentMetricsCommand, StreamDeploymentMetricsCommand
+    GetDeploymentMetricsCommand, StreamDeploymentMetricsCommand, DeploymentLogsCommand
+
+
+def get_workspace_handler(api_key):
+    logger_ = clilogger.CliLogger()
+    workspace_handler = workspace.S3WorkspaceHandlerWithProgressbar(api_key=api_key,
+                                                                    logger_=logger_,
+                                                                    uploader_cls=DeploymentWorkspaceDirectoryUploader,
+                                                                    client_name=CLI_PS_CLIENT_NAME)
+    return workspace_handler
+
+
+def validate_autoscaling_metric_or_resource(ctx, param, value, metric_type):
+    """
+    value in = ("cpu/targetAverage:10")
+    value out = ({"type": instance,
+                  "name": "cpu",
+                  "value_type": "targetAverage",
+                  "value": 10})
+    """
+
+    if value is None:
+        return None
+
+    old_values = value
+    new_values = []
+
+    for old_value in old_values:
+        try:
+            name, values = old_value.split("/", 1)
+            value_type, value = values.split(":", 1)
+            value = float(value)
+        except Exception as e:
+            debug_msg = "Error occurred while validating autoscaling {} with value {}: {}" \
+                .format(metric_type, old_value, e)
+            clilogger.CliLogger().debug(debug_msg)
+
+            msg = "value need to be in format resource_name/value_type:value for example cpu/targetAverage:60" \
+                .format(old_value)
+            raise click.BadParameter(msg)
+
+        new_value = {"type": metric_type,
+                     "name": name,
+                     "value_type": value_type,
+                     "value": value}
+        new_values.append(new_value)
+
+    return tuple(new_values)
+
+
+def validate_autoscaling_metric(ctx, param, value):
+    return validate_autoscaling_metric_or_resource(ctx, param, value, "Metric")
+
+
+def validate_autoscaling_resource(ctx, param, value):
+    return validate_autoscaling_metric_or_resource(ctx, param, value, "Resource")
 
 
 @cli.group("deployments", help="Manage deployments", cls=ClickGroup)
-def deployments():
+def deployments_group():
     pass
 
 
-@deployments.group("tags", help="Manage deployments tags", cls=ClickGroup)
+@deployments_group.group("tags", help="Manage deployments tags", cls=ClickGroup)
 def deployments_tags():
     pass
 
 
-@deployments.group(name="metrics", help="Read model deployment metrics", cls=ClickGroup)
+@deployments_group.group(name="metrics", help="Read model deployment metrics", cls=ClickGroup)
 def deployments_metrics():
     pass
 
 
-def get_deployment_client(api_key):
-    deployment_client = DeploymentsClient(api_key=api_key, logger=clilogger.CliLogger(), ps_client_name="gradient-cli")
-    return deployment_client
-
-
-@deployments.command("create", help="Create new deployment")
+@deployments_group.command("create", help="Create new deployment")
 @click.option(
     "--deploymentType",
     "deployment_type",
@@ -43,6 +95,12 @@ def get_deployment_client(api_key):
     help="Model deployment type",
     cls=common.GradientOption,
 )
+@click.option(
+    "--projectId",
+    "project_id",
+    help="Project ID",
+    cls=common.GradientOption,
+)
 @click.option(
     "--modelId",
     "model_id",
@@ -74,7 +132,6 @@ def get_deployment_client(api_key):
     "--instanceCount",
     "instance_count",
     type=int,
-    required=True,
     help="Number of machine instances",
     cls=common.GradientOption,
 )
@@ -114,12 +171,6 @@ def get_deployment_client(api_key):
     help="Container URL path",
     cls=common.GradientOption,
 )
-@click.option(
-    "--endpointUrlPath",
-    "endpoint_url_path",
-    help="Endpoint URL path",
-    cls=common.GradientOption,
-)
 @click.option(
     "--method",
     "method",
@@ -190,14 +241,72 @@ def get_deployment_client(api_key):
     help="Separated by comma tags that you want add to model deployment job",
     cls=common.GradientOption
 )
+@click.option(
+    "--workspace",
+    "workspace",
+    help="Path to workspace directory, archive, S3 or git repository",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--workspaceRef",
+    "workspace_ref",
+    help="Git commit hash, branch name or tag",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--workspaceUsername",
+    "workspace_username",
+    help="Workspace username",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--workspacePassword",
+    "workspace_password",
+    help="Workspace password",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--minInstanceCount",
+    "min_instance_count",
+    help="Minimal instance count",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--maxInstanceCount",
+    "max_instance_count",
+    help="Maximal instance count",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--scaleCooldownPeriod",
+    "scale_cooldown_period",
+    help="Scale cooldown period",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--metric",
+    "metrics",
+    multiple=True,
+    callback=validate_autoscaling_metric,
+    help="Autoscaling metrics. Example: my_metric/targetAverage:21.37",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--resource",
+    "resources",
+    multiple=True,
+    callback=validate_autoscaling_resource,
+    help="Autoscaling resources. Example: cpu/target:60",
+    cls=common.GradientOption,
+)
 @api_key_option
 @common.options_file
 def create_deployment(api_key, options_file, **kwargs):
     cliutils.validate_auth_options(kwargs)
     kwargs["tags"] = validate_comma_split_option(kwargs.pop("tags_comma"), kwargs.pop("tags"))
     del_if_value_is_none(kwargs)
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.CreateDeploymentCommand(deployment_client=deployment_client)
+    command = deployments_commands.CreateDeploymentCommand(api_key=api_key,
+                                                           workspace_handler=get_workspace_handler(api_key))
     command.execute(**kwargs)
 
 
@@ -214,7 +323,7 @@ DEPLOYMENT_STATES_MAP = collections.OrderedDict(
 )
 
 
-@deployments.command("list", help="List deployments with optional filtering")
+@deployments_group.command("list", help="List deployments with optional filtering")
 @click.option(
     "--state",
     "state",
@@ -245,15 +354,14 @@ DEPLOYMENT_STATES_MAP = collections.OrderedDict(
 @common.options_file
 def get_deployments_list(api_key, options_file, **filters):
     del_if_value_is_none(filters)
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.ListDeploymentsCommand(deployment_client=deployment_client)
+    command = deployments_commands.ListDeploymentsCommand(api_key=api_key)
     try:
         command.execute(**filters)
     except exceptions.ApplicationError as e:
         clilogger.CliLogger().error(e)
 
 
-@deployments.command("start", help="Start deployment")
+@deployments_group.command("start", help="Start deployment")
 @click.option(
     "--id",
     "id_",
@@ -264,12 +372,11 @@ def get_deployments_list(api_key, options_file, **filters):
 @api_key_option
 @common.options_file
 def start_deployment(id_, options_file, api_key=None):
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.StartDeploymentCommand(deployment_client=deployment_client)
+    command = deployments_commands.StartDeploymentCommand(api_key=api_key)
     command.execute(deployment_id=id_)
 
 
-@deployments.command("stop", help="Stop deployment")
+@deployments_group.command("stop", help="Stop deployment")
 @click.option(
     "--id",
     "id_",
@@ -280,12 +387,11 @@ def start_deployment(id_, options_file, api_key=None):
 @api_key_option
 @common.options_file
 def stop_deployment(id_, options_file, api_key=None):
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.StopDeploymentCommand(deployment_client=deployment_client)
+    command = deployments_commands.StopDeploymentCommand(api_key=api_key)
     command.execute(deployment_id=id_)
 
 
-@deployments.command("delete", help="Delete deployment")
+@deployments_group.command("delete", help="Delete deployment")
 @click.option(
     "--id",
     "id_",
@@ -296,12 +402,11 @@ def stop_deployment(id_, options_file, api_key=None):
 @api_key_option
 @common.options_file
 def delete_deployment(id_, options_file, api_key):
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.DeleteDeploymentCommand(deployment_client=deployment_client)
+    command = deployments_commands.DeleteDeploymentCommand(api_key=api_key)
     command.execute(deployment_id=id_)
 
 
-@deployments.command("update", help="Modify existing deployment")
+@deployments_group.command("update", help="Modify existing deployment")
 @click.option(
     "--id",
     "deployment_id",
@@ -316,6 +421,12 @@ def delete_deployment(id_, options_file, api_key):
     help="Model deployment type",
     cls=common.GradientOption,
 )
+@click.option(
+    "--projectId",
+    "project_id",
+    help="Project ID",
+    cls=common.GradientOption,
+)
 @click.option(
     "--modelId",
     "model_id",
@@ -347,6 +458,12 @@ def delete_deployment(id_, options_file, api_key):
     help="Number of machine instances",
     cls=common.GradientOption,
 )
+@click.option(
+    "--command",
+    "command",
+    help="Deployment command",
+    cls=common.GradientOption,
+)
 @click.option(
     "--containerModelPath",
     "container_model_path",
@@ -377,12 +494,6 @@ def delete_deployment(id_, options_file, api_key):
     help="Container URL path",
     cls=common.GradientOption,
 )
-@click.option(
-    "--endpointUrlPath",
-    "endpoint_url_path",
-    help="Endpoint URL path",
-    cls=common.GradientOption,
-)
 @click.option(
     "--method",
     "method",
@@ -433,16 +544,75 @@ def delete_deployment(id_, options_file, api_key):
     help="Cluster ID",
     cls=common.GradientOption,
 )
+@click.option(
+    "--workspace",
+    "workspace",
+    help="Path to workspace directory, archive, S3 or git repository",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--workspaceRef",
+    "workspace_ref",
+    help="Git commit hash, branch name or tag",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--workspaceUsername",
+    "workspace_username",
+    metavar="<username>",
+    help="Workspace username",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--workspacePassword",
+    "workspace_password",
+    help="Workspace password",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--minInstanceCount",
+    "min_instance_count",
+    help="Minimal instance count",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--maxInstanceCount",
+    "max_instance_count",
+    help="Maximal instance count",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--scaleCooldownPeriod",
+    "scale_cooldown_period",
+    help="Scale cooldown period",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--metric",
+    "metrics",
+    multiple=True,
+    callback=validate_autoscaling_metric,
+    help="Autoscaling metrics. Example: my_metric/targetAverage:21.37",
+    cls=common.GradientOption,
+)
+@click.option(
+    "--resource",
+    "resources",
+    multiple=True,
+    callback=validate_autoscaling_resource,
+    help="Autoscaling resources. Example: cpu/target:60",
+    cls=common.GradientOption,
+)
 @api_key_option
 @common.options_file
 def update_deployment(deployment_id, api_key, options_file, **kwargs):
     del_if_value_is_none(kwargs)
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.UpdateDeploymentCommand(deployment_client=deployment_client)
+    command = deployments_commands.UpdateDeploymentCommand(api_key=api_key,
+                                                           workspace_handler=get_workspace_handler(api_key))
     command.execute(deployment_id, **kwargs)
 
 
-@deployments.command("details", help="Get details of model deployment")
+@deployments_group.command("details", help="Get details of model deployment")
 @click.option(
     "--id",
     "deployment_id",
@@ -453,8 +623,7 @@ def update_deployment(deployment_id, api_key, options_file, **kwargs):
 @api_key_option
 @common.options_file
 def get_deployment(deployment_id, api_key, options_file):
-    deployment_client = get_deployment_client(api_key)
-    command = deployments_commands.GetDeploymentDetails(deployment_client=deployment_client)
+    command = deployments_commands.GetDeploymentDetails(api_key=api_key)
     command.execute(deployment_id)
 
 
@@ -484,9 +653,7 @@ def get_deployment(deployment_id, api_key, options_file):
 def deployment_add_tag(id, options_file, api_key, **kwargs):
     kwargs["tags"] = validate_comma_split_option(kwargs.pop("tags_comma"), kwargs.pop("tags"), raise_if_no_values=True)
 
-    deployment_client = get_deployment_client(api_key)
-
-    command = DeploymentAddTagsCommand(deployment_client=deployment_client)
+    command = DeploymentAddTagsCommand(api_key=api_key)
     command.execute(id, **kwargs)
 
 
@@ -516,9 +683,7 @@ def deployment_add_tag(id, options_file, api_key, **kwargs):
 def deployment_remove_tags(id, options_file, api_key, **kwargs):
     kwargs["tags"] = validate_comma_split_option(kwargs.pop("tags_comma"), kwargs.pop("tags"), raise_if_no_values=True)
 
-    deployment_client = get_deployment_client(api_key)
-
-    command = DeploymentRemoveTagsCommand(deployment_client=deployment_client)
+    command = DeploymentRemoveTagsCommand(api_key=api_key)
     command.execute(id, **kwargs)
 
 
@@ -567,8 +732,7 @@ def deployment_remove_tags(id, options_file, api_key, **kwargs):
 @api_key_option
 @common.options_file
 def get_deployment_metrics(deployment_id, metrics_list, interval, start, end, options_file, api_key):
-    deployment_client = get_deployment_client(api_key)
-    command = GetDeploymentMetricsCommand(deployment_client=deployment_client)
+    command = GetDeploymentMetricsCommand(api_key=api_key)
     command.execute(deployment_id, start, end, interval, built_in_metrics=metrics_list)
 
 
@@ -603,6 +767,37 @@ def get_deployment_metrics(deployment_id, metrics_list, interval, start, end, op
 @api_key_option
 @common.options_file
 def stream_model_deployment_metrics(deployment_id, metrics_list, interval, options_file, api_key):
-    deployment_client = get_deployment_client(api_key)
-    command = StreamDeploymentMetricsCommand(deployment_client=deployment_client)
+    command = StreamDeploymentMetricsCommand(api_key=api_key)
     command.execute(deployment_id=deployment_id, interval=interval, built_in_metrics=metrics_list)
+
+
+@deployments_group.command("logs", help="List deployment logs")
+@click.option(
+    "--id",
+    "deployment_id",
+    required=True,
+    cls=common.GradientOption,
+)
+@click.option(
+    "--line",
+    "line",
+    default=0,
+    cls=common.GradientOption,
+)
+@click.option(
+    "--limit",
+    "limit",
+    default=10000,
+    cls=common.GradientOption,
+)
+@click.option(
+    "--follow",
+    "follow",
+    default=False,
+    cls=common.GradientOption,
+)
+@api_key_option
+@common.options_file
+def list_logs(deployment_id, line, limit, follow, options_file, api_key=None):
+    command = DeploymentLogsCommand(api_key=api_key)
+    command.execute(deployment_id, line, limit, follow)
